/*
 *      idlescan
 *      Copyright (C) 1999 LiquidK <liquidk@superbofh.org>
 *
 */


#include "global.h"

typedef void (* packet_send_t)(int, int, int, int, int);
typedef int (* packet_check_t)(int, struct iphdr *, int, int, int);

void send_syn(int sock, int source_ip, int dest_ip, int dest_port);
void packet_send_icmpecho(int sock, int source, int dest, int id, int seq);
int packet_check_icmpecho(int sock, struct iphdr *ip, int size, int my_ip, int id);


